function [Angles,InfinitelyMany]=bbikt(robot,pos);
%IKT Inverzni kinematicka uloha.
%   [Angles,InfinitelyMany]=bbikt(robot,pos) vrati do Angles uhly prislusne 
%   motorum A az F (matice 4x6, kazdy radek jedno reseni), do InfinitelyMany vrati: 
%      0 - v Angles 4 reseni 
%      1 - v Angles 2 reseni, motor F lze volit (tento pripad nastane pokud 
%          pos(1) = 0 a pos(2) = 0)
%   pos ... [Point,Alpha,A,T]  kde 'Point' je bod (koncovy), 'Alpha' je rozevreni chapadla ve stupnich,
%           'A' je natoceni posledniho ramene od svisle osy, 'T' je natoceni posledniho ramene.
%   Promenna 'robot' udava parametry robota (viz INIT).
%
%  Reseni 1: robot zepredu, rameno nahoru
%         2: robot zepredu, rameno dolu
%         3: robot zezadu,  rameno nahoru
%         4: robot zezadu,  rameno dolu
InfinitelyMany = 0;
switch robot.robotname
case {'BB1','BB2'}   
   %Motor F
   if pos(2) == 0 
      if pos(1) > 0 
         Angles(1,6) = 90;
      elseif pos(1) < 0 
         Angles(1,6) =  -90;
      else
         Angles(1,6) = 0;
         InfinitelyMany = 1;
      end
   else
      Angles(1,6) = radtodeg(atan(pos(1)/pos(2)));
      if Angles(1,6) < 0 
         Angles(1,6) = Angles(1,6) + 180;
      end
      if (pos(1) < 0) | ((pos(1) == 0) & (pos(2) < 0)) 
         Angles(1,6) = Angles(1,6) + 180;
      end
   end
   Angles(2,6) = Angles(1,6);
   
   
   pos(1) =  sqrt(pos(1)^2+pos(2)^2);
   %dal se to pocita v rovine ramena, kolme k rovine X,Y
   AX = 0;AY = robot.L1;
   CX = pos(1)-robot.L4*sin(degtorad(pos(5)));CY = pos(3)+robot.L4*cos(degtorad(pos(5)));
   AC = sqrt((CX-AX)^2+(CY-AY)^2);
   AD = sqrt((pos(1)-AX)^2+(pos(3)-AY)^2);
   C0 = sqrt(CX^2+CY^2);
   S1 = (robot.L2+robot.L3+AC)/2;
   S2 = (robot.L1+AC+C0)/2;
   S3 = (robot.L4+AC+AD)/2;
   if (S1-robot.L2)*(S1-AC)/(S1*(S1-robot.L3)) < 0 
      Angles  = [];
      return;%Bod pos moc daleko od uchyceni
   end;
   
   
   %motor E
   if S2-C0 == 0 
      P1 = radtodeg(pi);
   else
      P1 = radtodeg(2*atan(sqrt((S2-AC)*(S2-robot.L1)/(S2*(S2-C0)))));
   end
   P2 = radtodeg(2*atan(sqrt((S1-robot.L2)*(S1-AC)/(S1*(S1-robot.L3)))));
   if CX < AX 
      P1 = -P1;
   end
   Angles(1,5) = P1+P2;
   Angles(2,5) = P1-P2;
   
   
   %motor D
   if S1-AC == 0
      P1 = radtodeg(pi);
   else
      P1 = radtodeg(2*atan(sqrt((S1-robot.L3)*(S1-robot.L2)/(S1*(S1-AC)))));
   end
   Angles(1,4) = P1;
   Angles(2,4) = -P1;
   
   
   %motor C
   if S3-AD == 0
      P1 = radtodeg(pi);
   else
      P1 = radtodeg(2*atan(sqrt((S3-robot.L4)*(S3-AC)/(S3*(S3-AD)))));
   end
   P2 = radtodeg(2*atan(sqrt((S1-robot.L3)*(S1-AC)/(S1*(S1-robot.L2)))));
   if distance(AX,AY,CX,CY,pos(1),pos(3)) < 0 
      P1 = -P1;
   end
   Angles(1,3) = P1+P2;
   Angles(2,3) = P1-P2;
   
   
   %motor B
   Angles(1,2) = pos(6);
   Angles(2,2) = pos(6);
   
   
   %motor A
   Angles(1,1) = pos(4);
   Angles(2,1) = pos(4);
   
   
   if ~InfinitelyMany
      Angles(3,6) = Angles(1,6) - 180;
      Angles(3,5) = -Angles(1,5);
      Angles(3,4) = -Angles(1,4);
      Angles(3,3) = -Angles(1,3);
      Angles(3,2) = Angles(1,2) - 180;
      Angles(3,1) = Angles(1,1);
      
      Angles(4,6) = Angles(2,6) - 180;
      Angles(4,5) = -Angles(2,5);
      Angles(4,4) = -Angles(2,4);
      Angles(4,3) = -Angles(2,3);
      Angles(4,2) = Angles(2,2) - 180;
      Angles(4,1) = Angles(2,1);
   end
   
   for i = 2:5,
      for j = 1:size(Angles,1),
         if Angles(j,i) < 0 
            Angles(j,i) = Angles(j,i) + 360;
         end
      end
   end
   
   for j = 1:size(Angles,1),
      if Angles(j,6) > 180
         Angles(j,6) = Angles(j,6) - 360;
      elseif Angles(j,6) <= -180
         Angles(j,6) = Angles(j,6) + 360;
      end
   end
   
case 'bosch'
   c=sqrt(pos(1)^2+pos(2)^2);
   b=acos((c^2-robot.L1^2-robot.L2^2)/(2*robot.L1*robot.L2))*180/pi;
   if imag(b) ~= 0 
      Angles=[];
      return
   end
   g=acos((c^2+robot.L1^2-robot.L2^2)/(2*robot.L1*c))*180/pi;
   if pos(1)==0
      d=90*sign(pos(2));
   else
      d=atan(pos(2)/pos(1))*sign(pos(1))*180/pi;
   end
   
   Angles(1,4)=pos(4);
   Angles(1,3)=-pos(3);
   Angles(1,2)=b;
   Angles(1,1)=90-d-g;
   Angles(2,4)=pos(4);
   Angles(2,3)=-pos(3);
   Angles(2,2)=-b;
   Angles(2,1)=90-d+g;
   
   if pos(1)~=0
      Angles(:,1:2)=Angles(:,1:2)*sign(pos(1));   
   end
   
   if pos(1) < 0
      p=Angles(1,:);
      Angles(1,:)=Angles(2,:);
      Angles(2,:)=p;
   end
end


function result=distance(AX,AY,BX,BY,CX,CY);
%DISTANCE Spocte v rovine orientovanou vzdalenost bodu C od primky AB.
%    result=distance(AX,AY,BX,BY,CX,CY);
UX = AX-CX;UY = AY-CY;
VX = AX-BX;VY = AY-BY;
result = (UX*VY-UY*VX)/sqrt(VX^2+VY^2);


