function [robot]=bbinit(robot);
%[robot]=bbinit(robot) - Robot initialization
%   Performs hardhome, softhome, and then 
%   returns robot settings in the output variable 'robot'.

% this is a patch which will recover the serial communication 
% from the read error after MARS 8 is switched on
h=robot.handle;
if mod(get(h,'ReadStatus'),2)==1
   fclose(robot.handle);
   [result, msg] = fopen(h);  % open serial port
   if result     % check if the port is open
      error(msg);                  % display error message
   end  
end 
%set timeout in seconds
robot=portsettimeout(robot,5);
portwriteline(robot,'ECHO:0');
pause(0.1); %wait until controller answers.
portflush(robot);

if isfield(robot,'REGPWRON') 
   portwriteline(robot,['REGPWRON:',num2str(robot.REGPWRON)]);
   disp('Press arm power button...');
   pause;
   portwriteline(robot,['REGPWRFLG:',num2str(robot.REGPWRFLG)]);
end

%Read specific robot constants. For details, see this function.
%eval(['robot=robot',robot.robotname,'(robot,0);']);

% stop any actions of the robot and purge any errors
bbresetmotors(robot);
bbisready(robot); %this function raise error if for example arm power is off
bbwaitforready(robot);

% initial hardware setting
bbsetspeed(robot, robot.defaultspeed);
bbsetacceleration(robot, robot.defaultacceleration);
for i = 1:6
   if robot.activemotors(i) ~= ' '
      portwriteline(robot,['REGME',robot.activemotors(i),':',num2str(robot.REGME(i))]);
   end
end
if isfield(robot,'REGCFG') 
   for i = 1:6
      if robot.activemotors(i) ~= ' '
         portwriteline(robot,['REGCFG',robot.activemotors(i),':',num2str(robot.REGCFG(i))]);
      end
   end
end
if isfield(robot,'REGP') 
   for i = 1:6
      if robot.activemotors(i) ~= ' '
         portwriteline(robot,['REGP',robot.activemotors(i),':',num2str(robot.REGP(i))]);
      end
   end
end
if isfield(robot,'REGI') 
   for i = 1:6
      if robot.activemotors(i) ~= ' '
         portwriteline(robot,['REGI',robot.activemotors(i),':',num2str(robot.REGI(i))]);
      end
   end
end
if isfield(robot,'REGD') 
   for i = 1:6
      if robot.activemotors(i) ~= ' '
         portwriteline(robot,['REGD',robot.activemotors(i),':',num2str(robot.REGD(i))]);
      end
   end
end

%do the hardhome
%for i = 1:6
%   if robot.activemotors(i) ~= ' '
%      portwriteline(robot,['HH',robot.activemotors(i),':']);
%   end
%end
%bbwaitforready(robot);

%do the softhome
%bbsofthome(robot);

%this flag indicate if bbinit has executed
robot.hhflag = 1;
return
