function u2 = u2udist(p,u1,SWAP_DISTFCN)
%u2udist  Radial lens distortion function applied for image points.
%         u2 = u2udist(p,u1), where
%         u1 ... Size (2,N). Undistorted image points.
%         u2 ... Size (2,N). Distorted image points.
%         p ... Size (?,1). Radial lens distortion parameters. See r2rdist.
%
%         See also UDIST2U, R2RDIST, RDIST2R.

%       Author       : (c) T. Werner and T. Pajdla, {werner,pajdla}@cmp.felk.cvut.cz
%                      07/09/98
%                      Center for Machine Perception, Czech Technical University, Prague
%       Documentation: ftp://cmp.felk.cvut.cz/pub/cmp/articles/werner/tr97-138.ps.gz     
%       Language     : Matlab 5.1, (c) MathWorks                         
%       Last change  : $Id: u2udist.m,v 1.1 1998/09/07 15:54:15 Matlab Exp $
%       Status       : Ready ($Source: /home.others/Matlab/Matlab/radial/RCS/u2udist.m,v $)


if size(p,1)<4, error('Bad size of parameter p.'); end
if exist('SWAP_DISTFCN') == 0
  SWAP_DISTFCN = 0;
end

u0 = p(1:2);
r0 = p(3);
k  = p(4:length(p)) ;         % extracting parameters from p
N  = size(u1,2);              % number of points
u0 = u0*ones(1,N);            % make u0 of the same size as u, ud
r1 = sqrt( sum((u1-u0).^2) ); % radius

u2 = u0 + (u1-u0).*([1;1]*(r2rdist(r0,k,r1,SWAP_DISTFCN)./r1));

return
